#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb  5 17:30:30 2024

@author: psakic

Low-level functions to import GAMIT metadata file as pandas DataFrame

"""

import datetime as dt
import re

import numpy as np
import pandas as pd

from rinexmod import logger as rimo_log
from rinexmod import rinexmod_api as rimo_api

logger = rimo_log.logger_define("INFO")


def read_gamit_apr_lfile(aprfile_inp):
    """
    read a GAMIT's apr/L-File (GNSS stations coordinates and DOMES)
    and store the data in a DataFrame

    Parameters
    ----------
    aprfile_inp : str
        path of the input apr/L-File

    Returns
    -------
    df : pandas DataFrame
        the apr/lfile info data

    Note
    ----
    If no DOMES is provided in the last 'Note' column,
    a dummy DOMES 00000X000 is returned

    site codes are given in lower case

    """
    lines_stk = []

    for l in open(aprfile_inp):

        if not l[0] == " " or l.strip().startswith("#") or l.strip().startswith("*"):
            continue

        # x, y, z = np.nan, np.nan, np.nan
        # vx,vy,vz = 0.,0.,0.
        # t = pd.NaT

        f = l[1:].split()

        site = l[1:5]
        try:
            site_full = f[0]
        except Exception as e:
            logger.error("unable to read the GAMIT's lfile")
            raise e

        x = float(f[1])
        y = float(f[2])
        z = float(f[3])
        if len(f) >= 8:
            ttmp = float(f[7])
        else:
            ttmp = 1980.0

        # if np.isclose(ttmp , 0.):
        #     t = conv.year_decimal2dt(2000.)
        # else:
        #     t = conv.year_decimal2dt(ttmp)
        t = ttmp

        # if len(l) > 225: # if velocities are given (125 is arbitrary)
        #     vX = float(f[8])
        #     vY = float(f[9])
        #     vZ = float(f[10])
        # else: # if no velocityis given
        #     vX = 0.
        #     vY = 0.
        #     vZ = 0.

        domes_re = re.search("[0-9]{5}[A-Z][0-9]{3}", l)
        if domes_re:
            domes = domes_re.group()
        else:
            domes = "00000X000"

        lines_stk.append((site, site_full, t, x, y, z, domes))

    df = pd.DataFrame(lines_stk)
    df.columns = ["site", "site_full", "epoch", "x", "y", "z", "domes"]

    df["site"] = df["site"].str.lower()

    return df


def read_gamit_station_info(station_info_inp, sort=True):
    """
    read a GAMIT's station.info (GNSS stations metadata)
    and store the data in a DataFrame

    Odd rows in station.info will be skipped

    Parameters
    ----------
    station_info_inp : str
        path of the input station.info.
    sort : bool, optional
        sort the DataFrame by site and start time.
        The default is True.

    Returns
    -------
    df : pandas DataFrame
        the station info data

    Note
    ----
    site codes are given in lower case
    """

    ### from gamit/lib/rstnfo.f
    #
    # c Label         Token  Format   Default      Description
    # c ------        -----  -----    ----------  ----------------------------------------------
    # c *SITE         sitcod  a4                  4-char station code
    # c Station Name  sname   a16                 16-char station name
    # c Session Start  start  2i4,3i3  0 0 0 0 0  start time (yr doy hr min sec)
    # c Session Stop  stop    2i4,3i3  0 0 0 0 0  stop time (yr doy hr min sec)
    # c Sessn         sessn   i1       0          session number
    # c Ant Ht        anth    f7.4     0.0        height of antenna ARP above the monument
    # c Ant N         antn    f7.4     0.0        N offset of ant ARP from the center of the monument
    # c Ant E         ante    f7.4     0.0        E offset of ant ARP from the center of the monument
    # c RcvCod        rcvcod  a6                  6-char/acter GAMIT receiver code
    # c Receiver Type rctype  a20                 IGS (RINEX) receiver name
    # c Receiver SN   rcvrsn  a20                 receiver serial number
    # c SwVer         swver   f5.2     0.0        GAMIT firmware code (real)
    # c Vers          rcvers  a20                 receiver firmware version from RINEX
    # c AntCod        antcod   a6                 6-charr GAMIT antenna code
    # c Antenna SN    antsn   a20                 20-char antenna serial number
    # c Antenna Type  anttyp  a15                 IGS antenna name, 1st 15 chars of RINEX antenna field
    # c Dome          radome   a5                 IGS radome name, last 5 chars of RINEX antenna field
    # c HtCod         htcod    a5     DHARP       5-char GAMIT code for type of height measurement
    # c AntDAZ	antdaz  f5.0     0.0        Alignment from True N (deg).  TAH 2020203.

    # colsize = np.array([4,16,4,4,3,3,3,4,4,3,3,3,1,7,7,7,6,20,20,5,20,6,20,15,5,5,5])
    colsize = np.array(
        [
            5,
            20,
            4 + 1,
            3 + 1,
            2 + 1,
            2 + 1,
            2 + 1,  # start
            4 + 1,
            3 + 1,
            2 + 1,
            2 + 1,
            2 + 1,  # end
            7 + 2,
            7 + 2,
            7 + 2,
            6 + 2,  # antenna high
            20 + 2,
            20 + 2,
            5 + 2,  # rec (2 version fields ...)
            20 + 2,  # rec SN
            15 + 2,
            5 + 2,
            20 + 2,
            1,  # Antenna
        ]
    )

    # colsize_use = colsize+1
    colsize_use = colsize
    # colsize = [e+1  for e in colsize]
    # colsize_use = colsize+2

    col = [
        "site",
        "station name",
        "start year",
        "start doy",
        "start hh",
        "start mm",
        "start ss",
        "stop year",
        "stop doy",
        "stop hh",
        "stop mm",
        "stop ss",
        "ant ht",
        "htcod",
        "ant n",
        "ant e",
        # 'rcvcod',
        "receiver type",
        "vers",
        "swver",
        "receiver sn",
        "antenna type",
        "dome",
        "antenna sn",
        "antdaz",
    ]

    bad_lines = []
    with open(station_info_inp, encoding="iso8859_1") as f:
        for il, l in enumerate(f.readlines()):
            if l[0] != " ":
                bad_lines.append(il)

    df = pd.read_fwf(
        station_info_inp,
        skiprows=bad_lines,
        widths=colsize_use,
        encoding="iso8859_1",
        header=None,
    )

    df.columns = col

    ##### clean df
    ### remove empty rows
    try:
        bool_empty_rows = df["site"].apply(len) == 1
    except Exception as e:
        logger.error("unable to read the GAMIT's station.info")
        raise e

    df = df[np.logical_not(bool_empty_rows)]
    ### do a second NaN cleaning, but the previous should have cleaned everything
    df.dropna(inplace=True, how="all")
    df.reset_index(inplace=True, drop=True)

    ##### create datetime start/end columns
    df["start year"] = df["start year"].replace(9999, 2099)
    df["start doy"] = df["start doy"].replace(999, 365)
    df["start hh"] = df["start hh"].replace(99, 00)
    df["start mm"] = df["start mm"].replace(99, 00)
    df["start ss"] = df["start ss"].replace(99, 00)

    df["stop year"] = df["stop year"].replace(9999, 2099)
    df["stop doy"] = df["stop doy"].replace(999, 365)
    df["stop hh"] = df["stop hh"].replace(99, 00)
    df["stop mm"] = df["stop mm"].replace(99, 00)
    df["stop ss"] = df["stop ss"].replace(99, 00)

    df_start = doy2dt(
        df["start year"],
        df["start doy"],
        df["start hh"],
        df["start mm"],
        df["start ss"],
    )

    df["start"] = df_start

    df_end = doy2dt(
        df["stop year"],
        df["stop doy"],
        df["stop hh"],
        df["stop mm"],
        df["stop ss"]
    )

    df["end"] = df_end

    df["site"] = df["site"].str.lower()

    df["start"] = pd.to_datetime(df["start"])
    df["end"] = pd.to_datetime(df["end"])

    if sort:
        df.sort_values(by=["site", "start"], inplace=True)

    return df


def gamit_df2instru_miscmeta(site, stinfo_df_inp, apr_df_inp, force_fake_coords=False):
    """
    read GAMIT files to get the RinexMod internal
    "instrus" and "misc_meta" dictionaries,
    necessary for the MetaData objects

    Here the site codes must remain as close as
    possible from the GAMIT input files
    i.e. 4 char.

    Parameters
    ----------
    site : str
        native site *4 char.* code which will be extracted from the
        station_info and apriori DataFrame.
    stinfo_df_inp : DataFrame
        station.info-like DataFrame.
    apr_df_inp : DataFrame
        lfile-like DataFrame.

    Returns
    -------
    installations : dict
        "installations" list i.e. list of "instrus" dict.
    mm_dic : dict
        "misc meta" dict.

    """
    #########################################################
    #### INSTRUMENTATION PART
    stinfo_df_site = stinfo_df_inp[stinfo_df_inp["site"] == site]
    installations = []

    for irow, row in stinfo_df_site.iterrows():
        inst_dic = dict()

        ##### dates
        inst_dic["dates"] = [row["start"].to_pydatetime(), row["end"].to_pydatetime()]

        ##### receiver
        rec_dic = dict()
        rec_dic["Receiver Type"] = str(row["receiver type"])
        rec_dic["Satellite System"] = "GPS+GLO+GAL+BDS+QZSS+SBAS"
        rec_dic["Serial Number"] = str(row["receiver sn"])
        rec_dic["Firmware Version"] = str(row["vers"])
        rec_dic["Elevation Cutoff Setting"] = "0"
        rec_dic["Date Installed"] = row["start"].to_pydatetime()
        rec_dic["Date Removed"] = row["end"].to_pydatetime()
        rec_dic["Temperature Stabiliz."] = "none"
        rec_dic["Additional Information"] = "none"

        inst_dic["receiver"] = rec_dic

        ##### antenna
        ant_dic = dict()
        ant_dic["Antenna Type"] = str(row["antenna type"])
        ant_dic["Serial Number"] = str(row["antenna sn"])
        ant_dic["Antenna Reference Point"] = "none"
        ant_dic["Marker->ARP Up Ecc. (m)"] = row["ant ht"]
        ant_dic["Marker->ARP North Ecc(m)"] = row["ant n"]
        ant_dic["Marker->ARP East Ecc(m)"] = row["ant e"]
        ant_dic["Alignment from True N"] = row["antdaz"]
        ant_dic["Antenna Radome Type"] = str(row["dome"])
        ant_dic["Radome Serial Number"] = "none"
        ant_dic["Antenna Cable Type"] = "none"
        ant_dic["Antenna Cable Length"] = "0"
        ant_dic["Date Installed"] = row["start"].to_pydatetime()
        ant_dic["Date Removed"] = row["end"].to_pydatetime()
        ant_dic["Additional Information"] = "none"
        ant_dic["metpack"] = "none"

        inst_dic["antenna"] = ant_dic

        installations.append(inst_dic)

    #########################################################
    #### MISC META PART
    apr_df_site = apr_df_inp[apr_df_inp["site"] == site]
    if len(apr_df_site) == 0 and not force_fake_coords:
        logger.error(
            "no coords in apr/lfile for %s, abort (you can force fake coords with -fc)",
            site,
        )
        raise rimo_api.RinexModError

    elif len(apr_df_site) == 0 and force_fake_coords:
        # logger.warning("no coords in apr/lfile for %s, fake coords at (0°,0°) used",site)
        apr_df_site = pd.Series(
            {"x": 6378137.000, "y": 0, "z": 0, "domes": "00000X000"}
        )
    else:
        apr_df_site = apr_df_site.iloc[-1]
        apr_df_site.squeeze()

    mm_dic = dict()

    mm_dic["IERS DOMES Number"] = apr_df_site["domes"]

    mm_dic["operator"] = "OPERATOR"
    mm_dic["agency"] = "AGENCY"

    mm_dic["X coordinate (m)"] = apr_df_site["x"]
    mm_dic["Y coordinate (m)"] = apr_df_site["y"]
    mm_dic["Z coordinate (m)"] = apr_df_site["z"]

    mm_dic["ID"] = site
    mm_dic["Country"] = "XXX"

    return installations, mm_dic


def doy2dt(year, days, hours=0, minutes=0, seconds=0):
    """
    Time representation conversion

    Day of Year Time => Python's datetime

    Parameters
    ----------
    year, days : float or list/numpy.array of floats.
        year, days of year
    hours, minutes, seconds : float or list/numpy.array of floats, optional
        hours, minutes, seconds

    Returns
    -------
    L : datetime or list/numpy.array of datetime.
        Datetime(s)

    Note
    ----
    extracted from the GeodeZYX toolbox
    https://github.com/IPGP/geodezyx-toolbox
    """
    if not is_iterable(year):
        # All this because Python cant handle int with a starting with 0 (like 08)
        # => SyntaxError: invalid token

        if np.any(np.isnan([year, days, hours, minutes, seconds])):
            logger.error(
                "one input is NaN, abort: %s,%s,%s,%s,%s",
                year,
                days,
                hours,
                minutes,
                seconds,
            )
            raise Exception

        try:
            year = int(float(str(year)))
            days = int(float(str(days)))
            hours = int(float(str(hours)))
            minutes = int(float(str(minutes)))
            seconds = int(float(str(seconds)))
        except Exception as e:
            logger.error(
                "error with conversion of %s,%s,%s,%s,%s",
                year,
                days,
                hours,
                minutes,
                seconds,
            )
            raise e

        tempsecs = seconds + 60 * minutes + 3600 * hours
        # finalsecs = np.floor(tempsecs)
        finalmicrosec = int(np.round(tempsecs * 10 ** 6))

        return (
                dt.datetime(year, 1, 1)
                + dt.timedelta(days - 1)
                + dt.timedelta(microseconds=finalmicrosec)
        )

    else:
        if not is_iterable(hours):
            hours = [0] * len(year)
        if not is_iterable(minutes):
            minutes = [0] * len(year)
        if not is_iterable(seconds):
            seconds = [0] * len(year)

        outlis = []
        typ = get_type_smart(year)
        for y, d, h, m, s in zip(year, days, hours, minutes, seconds):
            outlis.append(doy2dt(y, d, h, m, s))
        return typ(outlis)


def is_iterable(inp, consider_str_as_iterable=False):
    """
    Test if the input is an iterable like a list or a numpy array or not

    Parameters
    ----------
    inp : list, numpy.array, ...

    consider_str_as_iterable : bool
        string are considered as iterable by Python per default
        This boolean will avoid True as return if you test a string

    Returns
    -------
    out : bool
        True if inp is iterable, False either

    Note
    ----
    extracted from the GeodeZYX toolbox
    https://github.com/IPGP/geodezyx-toolbox
    """
    if not consider_str_as_iterable and type(inp) is str:
        return False

    try:
        iter(inp)
    except TypeError:
        out = False
    else:
        out = True
    return out


def get_type_smart(obj_in):
    """
    get type of an object, to convert easily another one to this type
    for instance type(np.array(A)) doesn't return a constructor

    Note
    ----
    extracted from the GeodeZYX toolbox
    https://github.com/IPGP/geodezyx-toolbox
    """
    typ = type(obj_in)
    if typ is np.ndarray:
        return np.array
    else:
        return typ



from . import logger
from . import rinexfile
from . import metadata
from . import gamit_meta
from . import rinexmod_api
from . import get_m3g

__version__='3.6.3' #  changed automaticcaly with bump-my-version




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This script takes a list of RINEX files (.d.Z or .d.gz or .rnx.gz),
and rename them according to the short <> long name convention.
The script recognize automatically the input convention and convert
it to the other one

See rinexrename -h for option description

2022-04-11  - Pierre Sakic & Félix Léger - leger@ipgp.fr

"""

from rinexmod.rinexfile import RinexFile
import shutil, os

def rinexrename(rinexinput, output=None, delete=False, alone=False, country="00XXX"):
    if alone:
        rinexinput = [rinexinput]
    elif isinstance(rinexinput, list):
        pass
    else:
        try:
            rinexinput = [line.strip() for line in open(rinexinput).readlines()]
        except:
            print('# ERROR : The input file is not a list : ' + rinexinput)
            return

    Output_path_list = []

    for rinex in rinexinput:

        RINEX_OBJ = RinexFile(rinex)

        if not output:
            output_dir = os.path.dirname(RINEX_OBJ.path)
        else:
            output_dir = output

        if RINEX_OBJ.name_conv == 'SHORT':
            new_name = RINEX_OBJ.get_longname(monum_country=country)
        elif RINEX_OBJ.name_conv == 'LONG':
            new_name = RINEX_OBJ.get_shortname()

        output_path = os.path.join(output_dir, new_name)

        shutil.copy(RINEX_OBJ.path, output_path)

        if delete:
            os.remove(RINEX_OBJ.path)

        Output_path_list.append(output_path)

    if alone:
        Output_path_list = Output_path_list[0]

    return Output_path_list


def main():
    import argparse


    # Parsing Args
    parser = argparse.ArgumentParser(description='rename a rinex file according to the short <> long name convention')
    parser.add_argument('rinexinput', type=str, help='Input rinex list file to process')
    parser.add_argument('-o', '--output', help='Output directory if different from input one',default=None)
    parser.add_argument('-d', '--delete', help='Delete input rinex file', action='store_true')
    parser.add_argument('-c', '--country', help='The monument/country code needed for the long name convention, e.g. 00FRA. 00XXX if not provided',default="00XXX")
    parser.add_argument('-a', '--alone', help='Input is a alone Rinex file and not a file containing list of Rinex files paths', action='store_true')

    args = parser.parse_args()

    rinexinput = args.rinexinput
    output = args.output
    delete = args.delete
    alone  = args.alone
    country = args.country

    rinexrename(rinexinput, output, delete, alone, country)

if __name__ == '__main__':
    main()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This program takes RINEX files (v2 or v3, compressed or not), rename them and 
modifiy their headers, and write them back to a destination directory. 

Check the program's synopsis (with --help option) or read the README 
for more details
(https://github.com/IPGP/rinexmod/blob/master/README.md)

# Credits:
v1 - 2021-02-07 Félix Léger - leger@ipgp.fr
v2 - 2023-03-23 Pierre Sakic - sakic@ipgp.fr
"""

import argparse
import textwrap

import rinexmod
import rinexmod.rinexmod_api as rimo_api

from argparse import RawTextHelpFormatter

def main():

    class SmartFormatter(argparse.HelpFormatter):
        # source: https://stackoverflow.com/a/22157136/3464212
        def _split_lines(self, text, width):
            if text.startswith("R|"):
                return text[2:].splitlines()
            # this is the RawTextHelpFormatter._split_lines
            return argparse.HelpFormatter._split_lines(self, text, width)

    # Parsing Args
    parser = argparse.ArgumentParser(
        description="rinexmod takes RINEX files (v2 or v3/4, compressed or not), "
        "rename them and modifiy their headers, and write them back to a destination directory",
        #formatter_class=SmartFormatter,
        formatter_class=RawTextHelpFormatter,
        epilog=textwrap.dedent(
            "RinexMod "
            + str(rinexmod.__version__)
            + " - GNU Public Licence v3 - P. Sakic et al. - IPGP-OVS - https://github.com/IPGP/rinexmod"
        ),
    )
    required = parser.add_argument_group("required arguments")
    optional = parser.add_argument_group("optional arguments")

    required.add_argument(
        "-i",
        "--rinexinput",
        type=str,
        required=True,
        nargs="+",
        help="Input RINEX file(s). It can be: \n"
        "1) a list file of the RINEX paths to process (generated with find or ls command for instance) \n"
        "2) several RINEX files paths \n"
        "3) a single RINEX file path (see -a/--alone for a single input file)",
    )
    required.add_argument(
        "-o",
        "--outputfolder",
        type=str,
        required=True,
        help="Output folder for modified RINEX files",
    )
    optional.add_argument(
        "-s",
        "--sitelog",
        help="Get the RINEX header values from file's site's sitelog."
        " Provide a single sitelog path or a folder contaning sitelogs.",
        type=str,
        default="",
    )

    optional.add_argument(
        "-k",
        "--modif_kw",
        help="Modification keywords for RINEX's header fields and/or filename.\n"
            "Format: -k keyword_1='value1' keyword2='value2'.\n"
            "Will override the information from the sitelog.\n"
            "Acceptable keywords: comment, marker_name, marker_number, station (legacy alias for marker_name), "
            "receiver_serial, receiver_type, receiver_fw, antenna_serial, antenna_type, "
            "antenna_X_pos, antenna_Y_pos, antenna_Z_pos, antenna_H_delta, "
            "antenna_E_delta, antenna_N_delta, operator, agency, sat_system, "
            "observables (legacy alias for sat_system), interval, "
            "filename_file_period (01H, 01D...), filename_data_freq (30S, 01S...), filename_data_source (R, S, U)",
        nargs="+",
        metavar="KEY=VALUE",
        action=rimo_api.ParseKwargs,
        default=None,
    )
    optional.add_argument(
        "-m",
        "--marker",
        help="A four or nine-character site code that will be used to rename input files."
        "(apply also to the header's MARKER NAME, but a custom -k marker_name='XXXX' overrides it)",
        type=str,
        default="",
    )
    optional.add_argument(
        "-co",
        "--country",
        help="A three-character string corresponding to the ISO 3166 Country code "
        "that will be used to rename input files. "
        "It overrides other country code sources (sitelog, --marker...). "
        "List of ISO country codes: https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes",
        type=str,
        default="",
    )
    optional.add_argument(
        "-n",
        "--ninecharfile",
        help="Path of a file that contains 9-char. site names (e.g. from the M3G database)",
        type=str,
        default="",
    )
    optional.add_argument(
        "-sti",
        "--station_info",
        help="Path of a GAMIT station.info file to obtain GNSS site metadata information (needs also -lfi option)",
        type=str,
        default="",
    )
    optional.add_argument(
        "-lfi",
        "--lfile_apriori",
        help="Path of a GAMIT apriori apr/L-File to obtain GNSS site position "
        "and DOMES information (needs also -sti option)",
        type=str,
        default="",
    )
    optional.add_argument(
        "-r",
        "--relative",
        help="Reconstruct files relative subfolders."
        "You have to indicate the common parent folder, "
        "that will be replaced with the output folder",
        type=str,
        default=0,
    )
    optional.add_argument(
        "-nh",
        "--no_hatanaka",
        help="Skip high-level RINEX-specific Hatanaka compression (performed per default). See also -c 'none'",
        action="store_true",
        default=False,
    )
    optional.add_argument(
        "-c",
        "--compression",
        type=str,
        help="Set low-level RINEX file compression "
        "(acceptable values : 'gz' (recommended to fit IGS standards), 'Z', 'none', default: %(default)s)",
        choices=['gz', 'Z', 'none'],
        default="gz",
    )
    optional.add_argument(
        "-l",
        "--longname",
        help="Force RINEX file renaming with long name convention (force gzip compression)."
        "Mutually exclusive with shortname.",
        action="store_true",
        default=False,
    )
    optional.add_argument(
        "-sh",
        "--shortname",
        help="Force RINEX file renaming with short name convention."
        "Mutually exclusive with longname.",
        action="store_true",
        default=False,
    )
    optional.add_argument(
        "-fs",
        "--force_sitelog",
        help="If a single sitelog is provided, force sitelog-based header values when RINEX's header and sitelog site"
        " name do not correspond. \n If several sitelogs are provided, skip badly-formated sitelogs.",
        action="store_true",
        default=False,
    )
    optional.add_argument(
        "-fc",
        "--force_fake_coords",
        help="When using GAMIT station.info metadata without apriori coordinates in the L-File, "
        "gives fake coordinates at (0°,0°) to the site",
        action="store_true",
        default=False,
    )
    optional.add_argument(
        "-fr",
        "--force_rnx_load",
        help="Force the loading of the input RINEX. Useful if its name is not standard",
        action="store_true",
        default=False,
    )
    optional.add_argument(
        "-ig",
        "--ignore",
        help="Ignore firmware changes between instrumentation periods when getting header values info from sitelogs",
        action="store_true",
    )
    optional.add_argument(
        "-a",
        "--alone",
        help="INPUT is a single/alone RINEX file (and not a list file of RINEX paths)",
        action="store_true",
    )
    optional.add_argument(
        "-ol",
        "--output_logs",
        help="Folder where to write output logs. If not provided, logs will be written to OUTPUTFOLDER",
        type=str,
    )
    optional.add_argument(
        "-w",
        "--write",
        help="Write (RINEX version, sample rate, file period) dependant output lists",
        action="store_true",
    )
    optional.add_argument(
        "-v",
        "--verbose",
        help="Print file's metadata before and after modifications.",
        action="store_true",
        default=False,
    )
    optional.add_argument(
        "-t",
        "--sort",
        help="Sort the input RINEX list.",
        action="store_true",
        default=False,
    )
    optional.add_argument(
        "-u",
        "--full_history",
        help="Add the full history of the station in the RINEX's 'header as comment.",
        action="store_true",
        default=False,
    )
    optional.add_argument(
        "-fns",
        "--filename_style",
        help="Set the RINEX filename style.\n"
        "acceptable values : 'basic' (per default), 'flex', 'exact'.\n"
        "* 'basic': a simple mode to apply a strict filename period (01H or 01D), "
        "being compatible with the IGS conventions.\n"
        "e.g.: FNG000GLP_R_20242220000_01D_30S_MO.crx.gz\n"
        "* 'flex': the filename period is tolerant and corresponds to"
        "the actual data content, but then can be odd (e.g. 07H, 14H...). "
        "The filename start time is rounded to the hour.\n"
        "e.g.: FNG000GLP_R_20242221800_06H_30S_MO.crx.gz\n"
        "* 'exact': the filename start time is strictly "
        "the one of the first epoch in the RINEX. "
        "Useful for some specific cases like splicing.\n"
        "e.g.: FNG000GLP_R_20242221829_06H_30S_MO.crx.gz\n"
        "(default: %(default)s)",
        choices=['basic', 'flex', 'exact'],
        default='basic',
    )
    optional.add_argument(
        "-mp",
        "--multi_process",
        help="Number of parallel multiprocesing (default: %(default)s, no parallelization)",
        type=int,
        default=1,
    )
    optional.add_argument(
        "-d",
        "--debug",
        help="Debug mode, stops if something goes wrong (default: %(default)s)",
        action="store_true",
        default=False,
    )

    optional.add_argument(
        "-rm",
        "--remove",
        help="Remove input RINEX file if the output RINEX is correctly written. Use it at your own risk. "
             "(default: %(default)s)",
        action="store_true",
        default=False,
    )

    optional.add_argument(
        "-krr",
        "--keep_rnx_rec",
        help="Keep the RINEX receiver header record in the output RINEX. "
             "Metadata from the external source (e.g. sitelogs) will not be modded. "
             "(default: %(default)s)",
        action="store_true",
        default=False,
    )

    args = parser.parse_args()

    rimo_api.rinexmod_cli(
        args.rinexinput,
        args.outputfolder,
        sitelog=args.sitelog,
        modif_kw=args.modif_kw,
        marker=args.marker,
        country=args.country,
        longname=args.longname,
        shortname=args.shortname,
        force_sitelog=args.force_sitelog,
        force_rnx_load=args.force_rnx_load,
        ignore=args.ignore,
        ninecharfile=args.ninecharfile,
        no_hatanaka=args.no_hatanaka,
        compression=args.compression,
        relative=args.relative,
        verbose=args.verbose,
        alone=args.alone,
        output_logs=args.output_logs,
        write=args.write,
        sort=args.sort,
        full_history=args.full_history,
        filename_style=args.filename_style,
        multi_process=args.multi_process,
        debug=args.debug,
        station_info=args.station_info,
        lfile_apriori=args.lfile_apriori,
        force_fake_coords=args.force_fake_coords,
        remove=args.remove,
        keep_rnx_rec=args.keep_rnx_rec
    )

if __name__ == '__main__':
    main()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 27/08/2024 20:14:09

@author: psakic
"""

import requests
import subprocess
import hashlib
import os, glob
import shutil
import sys

def get_m3g_sitelogs(
    sitelogsfolder,
    delete,
    observatory=None,
    root_folder=False,
    svn_mode=False,
    move_folder=None,
    force=False,
    exclude=[],
):
    """
    Downloads the latest version of sitelogs from the M3G repository and writes them
    to an observatory-dependent subfolder set in 'observatories'.

    Parameters
    ----------
    sitelogsfolder : str
        The folder where the downloaded sitelogs will be stored.
    delete : bool
        If True, deletes old sitelogs in the storage folder to keep only the latest version.
    observatory : str, optional
        IPGP's observatory ID to filter sitelogs to download.
        Valid values are: OVSM, OVSG, OVPF, REVOSIMA. Default is None.
    root_folder : bool, optional
        If True, stores sitelogs in the root of OUTPUTFOLDER. Default is False.
    svn_mode : bool, optional
        If True, maintains the legacy OVS's SVN folders,
        downloads the sitelog of a single observatory, and performs an 'svn commit'.
        Default is False.
    move_folder : str, optional
        The folder where old sitelogs will be moved. Default is None.
    force : bool, optional
        If True, forces download even if an identical sitelog already exists locally.
        Default is False.
    exclude : list, optional
        List of site codes (4 or 9 characters) to exclude from download.
        Default is an empty list.

    Returns
    -------
    None
    """
    # the root folder option is automatically activated for the SVN mode
    if svn_mode:
        root_folder = True

    # a single observatory must be given for the SVN mode
    if svn_mode and not observatory:
        sys.exit(
            "when using SVN mode (-s), a single observatory must be given with -o option"
        )

    # M3G country webservice. We use countries to filter IPGP's obs stations.
    country_url = "https://gnss-metadata.eu/v1/sitelog/metadata-list?country="
    country_url = "https://gnss-metadata.eu/v1/sitelog/metadata-list?network="

    # # Observatories. First field is the counrty code in M3G's webservice.
    # # Second field is the name of the observatory and is used to build the subfolder
    # observatories = {
    #     "MTQ": "OVSM",
    #     "GLP": "OVSG",
    #     "BLM": "OVSG",  ### St Barthelemy
    #     "MAF": "OVSG",  ### St Martin (no station there for the moment)
    #     "REU": "OVPF",
    #     "MYT": "REVOSIMA",
    #     "ATF": "REVOSIMA",
    #     "DJI": "OGA",
    # }  ### TAAF aka Terres Australes

    #(No more country but network 2025-03)
    observatories = {
        "MQ": "OVSM",
        "GL": "OVSG",
        "PF": "OVPF",
        "QM": "REVOSIMA",
        "DJ": "OGA",
    }


    # If an observatory ID is given, only download its sitelogs

    if observatory in observatories.values():
        observatories_clean = dict()
        for obs_key, obs_val in observatories.items():
            if obs_val in observatory:
                observatories_clean[obs_key] = obs_val

        observatories = observatories_clean

        ## simple way to filter observatories if observatories is bijective
        ## not the case anymore because ATF, BLM ... (2022-10)
        # obs_key = list(observatories.keys())[list(observatories.values()).index(observatory)]
        # observatories = {obs_key: observatories[obs_key]}

    # Check that output folder exists
    if not os.path.exists(sitelogsfolder):
        print("# ERROR : The output folder for sitelogs doesn't exist")
        return

    if move_folder and not os.path.exists(move_folder):
        print("# ERROR : The archive folder for old sitelogs doesn't exist")
        return

    # Check that obs' subfolder exists. If not, creates.
    for obs in [*observatories.values()]:
        if not root_folder:
            obs_path = os.path.join(sitelogsfolder, obs)
        else:
            obs_path = sitelogsfolder

        if not os.path.exists(obs_path):
            os.mkdir(obs_path)

    sitelog_local_paths = []

    for ctry in observatories.keys():

        obs_url = country_url + ctry

        obs_infos = requests.get(obs_url)
        obs_infos = obs_infos.content.decode("utf-8")
        obs_infos = obs_infos.splitlines()
        obs_infos = obs_infos[1:]  # remove header
        obs_infos = list(reversed(obs_infos))  # list is reversed per def.

        if not root_folder:
            obs_path = os.path.join(sitelogsfolder, observatories[ctry])
        else:
            obs_path = sitelogsfolder

        # If delete, empty folders
        if delete and not move_folder:
            old_sitelogs_del = glob.glob(obs_path + "/*" + ctry + "*.log")
            for f in old_sitelogs_del:
                os.remove(f)

        print(
            "###### Downloading {} ({}) sitelogs from M3G to {}".format(
                observatories[ctry], ctry, obs_path
            )
        )

        for line in obs_infos:
            line = line.split()

            # station_name = line[0]

            # If station is declared in M3G but sitelog not available yet
            if len(line) < 6:
                print("### " + line[0] + " : not available ###")
                continue

            sitelog_md5 = line[1]
            sitelog_url = line[5]
            sitelog_name = line[2]
            sitelog_local_path = os.path.join(obs_path, sitelog_name)
            sitelog_local_paths.append(sitelog_local_path)

            ### skip excluded stations
            exclude = [e.lower() for e in exclude]
            if sitelog_name[:4] in exclude or sitelog_name[:9] in exclude:
                print("### " + sitelog_name + " skip (excluded) ###")
                continue

            ### get the checksum for the existing sitelog, if any
            if os.path.exists(sitelog_local_path):
                local_file = open(sitelog_local_path, "rb")
                local_md5 = hashlib.md5(local_file.read()).hexdigest()
            else:
                local_md5 = None

            if force or (local_md5 != sitelog_md5):
                print("### " + sitelog_name + " download ###")
                ##Dowload the sitelog
                r = requests.get(sitelog_url, allow_redirects=True)
                # print(r.status_code)
                content = r.content  # .rstrip()
                open(sitelog_local_path, "wb").write(content)
                # subprocess.call(['wget',
                #                  '--no-check-certificate',
                #                  sitelog_url,'-q',
                #                  '-O', sitelog_local_path])

            else:
                print("### " + sitelog_name + " skip (already exists) ###")

            ### get existing old sitelogs for moving
            if move_folder:
                old_sitelogs_mv = glob.glob(
                    obs_path + "/*" + sitelog_name[:9] + "*.log"
                )
                if sitelog_local_path in old_sitelogs_mv:
                    old_sitelogs_mv.remove(sitelog_local_path)
                for f in old_sitelogs_mv:
                    print("### " + os.path.basename(f) + " moved to archive folder ###")
                    shutil.move(f, move_folder)

    if svn_mode:
        print("### SVN add/commit of the downloaded sitelogs")
        for sitelog_local in sitelog_local_paths:
            subprocess.call(["svn", "add", sitelog_local])
        subprocess.call(
            ["svn", "commit", "-m", "get_m3g_sitelogs auto commit", sitelogsfolder]
        )

    # # Other tecnhique using node webservice that will send back less information
    # # about the sitelogs but will filter with the node ID, and will return all
    # # sitelogs belonging to IPGP node
    #
    # node_url = "https://gnss-metadata.eu/v1/node/view?id="
    # node_id = '5e4d07d9468524145a7cf0f2' # IPGP
    #
    # sitelog_url = "https://gnss-metadata.eu/v1/sitelog/exportlog?id="
    #
    # # Getting station list related to IPGP node
    # node_url = node_url + node_id
    #
    # node_infos = requests.get(node_url)
    # node_infos = node_infos.content.decode('utf-8')
    # node_infos = json.loads(node_infos)
    #
    # station_list = node_infos['stations']
    #
    # for station in station_list:
    #
    #     output = os.path.join(sitelogsfolder, observatories[station[-3:]], station.lower() + '.log')

